#ifndef ACCOUNT_H
#define ACCOUNT_H

#define FACTOR 10000.0

#include <QMutex>
#include <QObject>

class Account : public QObject
{
	Q_OBJECT

public:
	Account(QThread *thread);
	~Account();

	static QString formatCurrency(qint64 amount);

	struct Data {
		int id;
		QString number;
		QString name;
		qint64 amount;
	};

	void clearData();

	int getSize();
	Data getAccount(int index = -1);
	Data getAccount(int id, bool fromDatabase);
	Data getAccount(QString number);
	int getActiveIndex();

	void setActiveIndex(int index);

signals:
	void finished();
	void emitLoadBalance(QString);

public slots:
	void process();
	void getAccounts(int);
	void updateActiveIndex();

private:
	QMutex m_mutex;

	int m_activeIndex;
	QVector<Data> *m_data;
};

#endif
