#include <QCryptographicHash>
#include <QSqlError>
#include <QSqlQuery>
#include <QThread>
#include <QVariant>

#include <QtDebug>

#include "card.h"
#include "db.h"

Card::Card(QThread *thread) :
	m_id(""),
	m_userId(0)
{
	this->moveToThread(thread);
	QObject::connect(thread,	&QThread::started,	this,	&Card::process);
	QObject::connect(this,		&Card::finished,	thread,	&QThread::quit);
	QObject::connect(this,		&Card::finished,	this,	&Card::deleteLater);
	QObject::connect(thread,	&QThread::finished,	thread,	&QThread::deleteLater);
}

Card::~Card() {
}

void Card::process() {
}

void Card::login(QString cardId, QString pinNumber) {
	QSqlQuery query = Db::q("SElECT * FROM `card` WHERE `id`=?", {cardId});
	if (query.exec() && query.size() == 1 && query.next()) {
		if (query.value("blocked").toInt() == 0) {

			QByteArray hash = QCryptographicHash::hash(
					cardId.toUtf8() + pinNumber.toUtf8(),
					QCryptographicHash::Sha256).toHex().data();
			int failedAttempts = query.value("failed_login_attempts").toInt();
			if (query.value("password").toString() == QString(hash)) {
				setId(cardId);
				setUserId(query.value("user_id").toInt());

				emit emitNewLog(Log::LOGIN, "", 1, "", "");

				emit loginUser(getUserId());
#ifdef DEBUG
				printf("Card, login: %s\n", query.value("id").toByteArray().data());
#endif
			}
			else if (failedAttempts < 2) {
				emit emitNewLog(Log::LOGIN, "failed", 0, "", "");

				QString message = "Invalid pin, " +
					QString::number(2 - failedAttempts) + " attempts remaining.";
				emit emitWrongPin(true, message);
				incrementFailedLogin(cardId, failedAttempts);
			}
			else {
				emit emitNewLog(Log::LOGIN, "blocked", 0, "", "");

				emit emitWrongPin(true, "Scanned card is blocked.");
				incrementFailedLogin(cardId, failedAttempts);
				blockUser(cardId, 1);
			}
		}
		else {
			emit emitWrongPin(true, "Scanned card is blocked.");
#ifdef DEBUG
			printf("Card, blocked\n");
#endif
		}
	}
	else {
		emit emitWrongPin(true, "Scanned card is unregistered.");
#ifdef DEBUG
		printf("Card, error: %d, %s\n",
				query.lastError().type(),
				query.lastError().text().toUtf8().data());
#endif
	}
}

void Card::incrementFailedLogin(QString cardId, int currentFails) {
#ifdef DEBUG
	printf("Card, failed login atttempt\n");
#endif

	QSqlQuery query = Db::q("UPDATE `card` SET `failed_login_attempts`=? WHERE `id`=?",
			{currentFails + 1, cardId});
	query.exec();
}

void Card::blockUser(QString cardId, int block) {
#ifdef DEBUG
	printf("Card, blocked\n");
#endif

	QSqlQuery query = Db::q("UPDATE `card` SET `blocked`=? WHERE `id`=?",
			{block, cardId});
	query.exec();
}

QString Card::getId() {
	QMutexLocker locker(&m_mutex);
	return m_id;
}

int Card::getUserId() {
	QMutexLocker locker(&m_mutex);
	return m_userId;
}

void Card::setId(QString id) {
	m_mutex.lock();
	m_id = id;
	m_mutex.unlock();
}

void Card::setUserId(int userId) {
	m_mutex.lock();
	m_userId = userId;
	m_mutex.unlock();
}
