#include "card.h"
#include "loginpage.h"
#include "mainwindow.h"
#include "serialcommand.h"
#include "ui_mainwindow.h"
#include "user.h"

LoginPage::LoginPage() {
}

LoginPage::~LoginPage() {
}

void LoginPage::setup(MainWindow *mainWindow, Card *card, User *user) {
	m_mainWindow = mainWindow;
	m_card = card;
	m_user = user;

	connect(m_card, &Card::emitWrongPin, this, &LoginPage::wrongPin);
	connect(m_user, &User::emitLogin, this, &LoginPage::userLogin);

	connect(m_mainWindow->getUi()->pinDisplay, &QLineEdit::selectionChanged, this, &LoginPage::deselectPinDisplay);
}

void LoginPage::onFocus() {
	// Reset ui
	cardDetected(false);
	pinEntered(0);
	wrongPin(false, "");
}

void LoginPage::deselectPinDisplay() {
	m_mainWindow->getUi()->pinDisplay->deselect();
}

void LoginPage::cardDetected(bool detected) {
	m_mainWindow->getUi()->cardLabel->setDisabled(detected);
	m_mainWindow->getUi()->pinLabel->setDisabled(!detected);
	m_mainWindow->getUi()->pinDisplay->setDisabled(!detected);
}

void LoginPage::pinEntered(int length) {
	QString pin;
	for (int i = 0; i < length; i++) {
		pin.append("●");
	}

	m_mainWindow->getUi()->pinDisplay->setText(pin);
}

void LoginPage::wrongPin(bool wrong, QString message) {
	if (wrong) {
		m_mainWindow->getUi()->pinDisplay->setStyleSheet("border: 1px solid red; \
				border-radius: 5px;");
		m_mainWindow->getUi()->pinMessage->setText(message);
	}
	else {
		m_mainWindow->getUi()->pinDisplay->setStyleSheet("border: 1px solid rgb(48, 52, 64); \
				border-radius: 5px;");
		m_mainWindow->getUi()->pinMessage->setText("\u00AD");
	}
}

void LoginPage::userLogin() {
	m_mainWindow->setWindowState(MainWindow::HOME_PAGE);
}
