/*
#include "card.h"
int main() { return 0; }
*/
#include <QApplication>
#include <QPushButton>
#include <QShortcut>

#include "account.h"
#include "card.h"
#include "db.h"
#include "log.h"
#include "mainwindow.h"
#include "serial.h"
#include "serialcommand.h"
#include "transaction.h"
#include "user.h"

int main(int argc, char *argv[]) {
	QApplication a(argc, argv);

#ifdef DEBUG
	printf("\n");
	printf("Main thread: %p\n\n", QThread::currentThreadId());
#endif

	// Database thread
	QThread *dthread = new QThread;
	Db *db = new Db(dthread);
	Card *card = new Card(dthread);
	User *user = new User(dthread);
	Account *account = new Account(dthread);
	Log *log = new Log(dthread, card);
	Transaction *transaction = new Transaction(dthread, account);

	// Serial thread
	QThread *sthread = new QThread;
	Serial *serial = new Serial(sthread);
	SerialCommand *serialCommand = new SerialCommand(sthread, card, user, account, log, transaction);

	MainWindow w(serialCommand, card, user, account, log, transaction);
	w.show();

	// Connections between threads
	QObject::connect(serial, &Serial::sendData, serialCommand, &SerialCommand::processData);
	QObject::connect(serialCommand, &SerialCommand::loginCard, card, &Card::login);
	QObject::connect(card, &Card::loginUser, user, &User::login);
	QObject::connect(user, &User::setAccounts, account, &Account::getAccounts);

	sthread->start();
	dthread->start();

	return a.exec();
}
