#include "mainwindow.h"
#include "serialcommand.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(SerialCommand *serialCommand, Card *card, User *user,
		Account *account, Log *log, Transaction *transaction) :
	m_ui(new Ui::MainWindow),
	m_log(log),
	m_afterLogin(false)
{
	qRegisterMetaType<MainWindow::WindowState>("MainWindow::WindowState");

	m_ui->setupUi(this);
	m_ui->loginPage->setup(this, card, user);
	m_ui->homePage->setup(this, user, account);
	m_ui->accountPage->setup(this, account);
	m_ui->balancePage->setup(this, account);
	m_ui->withdrawPage->setup(this, account, transaction);
	m_ui->transferPage->setup(this, account, transaction);
	serialCommand->setup(this);

	setWindowState(MainWindow::LOGIN_PAGE);

	connect(this, &MainWindow::emitNewLog, m_log, &Log::newLog);
}

MainWindow::~MainWindow() {
	delete m_ui;
}

Ui::MainWindow *MainWindow::getUi() {
	QMutexLocker locker(&m_mutex);
	return m_ui;
}

MainWindow::WindowState MainWindow::getWindowState() {
	QMutexLocker locker(&m_mutex);
	return m_windowState;
}

void MainWindow::setWindowState(MainWindow::WindowState windowState) {
	if (!windowState == MainWindow::LOGIN_PAGE && m_afterLogin) {
		emit emitNewLog(Log::PAGE_VISIT, QString::number((int)windowState), 1, "", "");
	}

	if (windowState == MainWindow::LOGIN_PAGE) {
		m_ui->loginPage->onFocus();

		m_afterLogin = false;
	}
	else if (windowState == MainWindow::HOME_PAGE) {
		m_ui->homePage->onFocus();

		if (!m_afterLogin) {
			m_afterLogin = true;
		}
	}
	else if (windowState == MainWindow::ACCOUNT_PAGE) {
		m_ui->accountPage->onFocus();
	}
	else if (windowState == MainWindow::BALANCE_PAGE) {
		m_ui->balancePage->onFocus();
	}
	else if (windowState == MainWindow::WITHDRAW_PAGE) {
		m_ui->withdrawPage->onFocus();
	}
	else if (windowState == MainWindow::TRANSFER_PAGE) {
		m_ui->transferPage->onFocus();
	}

	m_mutex.lock();
	m_windowState = windowState;
	m_mutex.unlock();

	m_ui->stackedWidget->setCurrentIndex(m_windowState);
}
