#include <stdlib.h>
#include <time.h>

#include <QThread>

#include <QtDebug>

#include "serial.h"

Serial::Serial(QThread *thread) :
	m_connected(false)
{
	srand (time(NULL));

	this->moveToThread(thread);
	QObject::connect(thread,	&QThread::started,	this,	&Serial::process);
	QObject::connect(this,		&Serial::finished,	thread,	&QThread::quit);
	QObject::connect(this,		&Serial::finished,	this,	&Serial::deleteLater);
	QObject::connect(thread,	&QThread::finished,	thread,	&QThread::deleteLater);
}

Serial::~Serial() {
	printf("Serial, cleaning up..\n");
	m_serialPort->close();
	delete m_serialPort;
}

void Serial::process() {
#ifdef DEBUG
	printf("Serial, thread: %p\n", QThread::currentThreadId());
#endif

	if (initSerial() == true) {
		m_mutex.lock();
		m_connected = true;
		m_mutex.unlock();

		connect(m_serialPort, &QSerialPort::readyRead, this, &Serial::readData);
		connect(m_serialPort, &QSerialPort::errorOccurred, this, &Serial::processError);
	}
	else {
		emit finished();
	}
}

void Serial::readData() {
	m_mutex.lock();
	m_data.append(m_serialPort->readAll());
	emit sendData(m_data);
	m_data.clear();
	m_mutex.unlock();
}

void Serial::processError() {
	if (m_serialPort->error() != QSerialPort::NoError &&
			m_serialPort->error() != QSerialPort::TimeoutError) {
#ifdef DEBUG
		printf("Serial, error: %d\n", m_serialPort->error());
#endif
		emit finished();
	}
}

bool Serial::initSerial() {
#ifdef DEBUG
	#if defined Q_OS_LINUX
		printf("Linux!\n");
	#elif defined Q_OS_MACOS
		prinrt("macOS!\n");
	#elif defined Q_OS_WIN
		printf("Windows!\n");
	#endif
#endif
	const auto infos = QSerialPortInfo::availablePorts();
	for (const QSerialPortInfo &info : infos) {
		// printf("------\n");
		// printf("%s\n", info.description().toUtf8().data());
		// printf("%s\n", info.manufacturer().toUtf8().data());
		// printf("%s\n", info.serialNumber().toUtf8().data());
		// printf("%d\n", (int)info.vendorIdentifier());
		// printf("%d\n", (int)info.productIdentifier());
		// printf("-----\n");
		if (info.vendorIdentifier() == VENDOR_IDENTIFIER
				&& info.productIdentifier() == PRODUCT_IDENTIFIER) {
			m_serialPortName = info.portName();
		}
	}
#ifdef DEBUG
	printf("Serial, port name: %s\n", m_serialPortName.toUtf8().data());
#endif
	m_serialPort = new QSerialPort();
	m_serialPort->setPortName(m_serialPortName);
	m_serialPort->setBaudRate(SERIAL_BAUD_RATE);

	if (!m_serialPort->open(QIODevice::ReadWrite)) {
#ifdef DEBUG
		printf("Serial, failed to open port: %s\n", m_serialPortName.toUtf8().data());
#endif
		return false;
	}

	return true;
}

bool Serial::getConnected() {
	QMutexLocker locker(&m_mutex);
	return m_connected;
}
