#ifndef SERIAL_COMMAND_H
#define SERIAL_COMMAND_H

#include <QMutex>
#include <QObject>

#include "log.h"
#include "mainwindow.h"

class Account;
class Card;
class User;
class Transaction;

class SerialCommand : public QObject
{
	Q_OBJECT

public:
	SerialCommand(QThread *thread, Card *card, User *user, Account *account, Log *log, Transaction *transaction);
	~SerialCommand();
	void setup(MainWindow *mainWindow);

	enum CommandType {
		CARD_DETECTED,
		CARD_ID,
		PIN,
	};

	void attemptLogin();

	void handleLoginPage(char input);
	void handleHomePage(char input);
	void handleAccountPage(char input);
	void handleBalancePage(char input);
	void handleWithdrawPage(char input);
	void handleTransferPage(char input);

signals:
	void finished();

	// Ui signals
	void emitWindowState(MainWindow::WindowState);
	void cardDetected(bool);
	void pinEntered(int);
	void emitNumberEntered(qint64);
	void emitSelectAccount(int);
	void emitSelectElement(bool);

	// Db signals
	void loginCard(QString, QString);
	void emitNewLog(Log::Type, QString, int, QString, QString);
	void emitTransfer(qint64, QString);

public slots:
	void process();
	void processData(QByteArray);

private:
	QMutex m_mutex;

	bool m_cardDetected;
	QString m_cardId;
	QString m_pin;
	int m_transferAccountSelected;

	Card *m_card;
	User *m_user;
	Account *m_account;
	Log *m_log;
	Transaction *m_transaction;
	MainWindow *m_mainWindow;

};

#endif
