#include <QSqlQuery>
#include <QThread>

#include "account.h"
#include "db.h"
#include "transaction.h"

Transaction::Transaction(QThread *thread, Account *account) :
	m_account(account)
{
	this->moveToThread(thread);
	QObject::connect(thread, &QThread::started, this, &Transaction::process);
	QObject::connect(this, &Transaction::finished, thread, &QThread::quit);
	QObject::connect(this, &Transaction::finished, this, &Transaction::deleteLater);
	QObject::connect(thread, &QThread::finished, thread, &QThread::deleteLater);
}

Transaction::~Transaction() {
}

void Transaction::process() {
}

bool Transaction::withdraw(qint64 amount, int id, bool add) {
	Account::Data account = m_account->getAccount(id, true);

	if (account.id != 0 && account.amount >= amount) {
		QString modifier = add
			? "UPDATE `account` SET `amount`=(`amount` + ?) WHERE `id`=?"
			: "UPDATE `account` SET `amount`=(`amount` - ?) WHERE `id`=?";
		QSqlQuery query = Db::q(modifier, {amount, account.id});

		if (query.exec()) {
			return true;
		}
	}

	return false;
}

void Transaction::transfer(qint64 amount, QString to) {
	amount *= FACTOR;

	if (m_account->getActiveIndex() != -1 && amount >= 0) {
		Account::Data account = m_account->getAccount(m_account->getActiveIndex());

		// Withdraw
		if (to.isEmpty()) {
			if (withdraw(amount, account.id)) {
				emit emitNewLog(Log::TRANSACTION, QString::number(amount / FACTOR), 1, account.number, "");
				emit emitWithdrawAttempt(true, "Balance subtracted.");
#ifdef DEBUG
				printf("Transaction, balance subtracted\n");
#endif
			}
			else {
				emit emitNewLog(Log::TRANSACTION, QString::number(amount / FACTOR), 0, account.number, "");
				emit emitWithdrawAttempt(false, "Not enough balance on account.");
			}
		}
		// Transfer
		else  if (account.number != to) {
			if (withdraw(amount, m_account->getAccount(m_account->getActiveIndex()).id)) {
				withdraw(amount, m_account->getAccount(to).id, true);
				emit emitNewLog(Log::TRANSACTION, QString::number(amount / FACTOR), 1, account.number, to);
				emit emitTransferAttempt(true, "Balance transfered");
#ifdef DEBUG
				printf("Transaction, balance transfered\n");
#endif
			}
			else {
				emit emitNewLog(Log::TRANSACTION, QString::number(amount / FACTOR), 0, account.number, to);
				emit emitTransferAttempt(false, "Not enough balance on account.");
			}
		}
		else {
			emit emitTransferAttempt(false, "From and to account are the same.");
		}
	}
}
