#include "account.h"
#include "mainwindow.h"
#include "transaction.h"
#include "ui_mainwindow.h"
#include "transferpage.h"

TransferPage::TransferPage() {
}

TransferPage::~TransferPage() {
}

void TransferPage::setup(MainWindow *mainWindow, Account *account, Transaction *transaction) {
	m_mainWindow = mainWindow;
    m_account = account;
	m_transaction = transaction;

	connect(m_transaction, &Transaction::emitTransferAttempt, this, &TransferPage::transferAttempt);
}

void TransferPage::onFocus() {
	// Reset ui
    loadAccounts();
	selectElement(false);
	selectAccount(0);
	numberEntered(0);
}

void TransferPage::loadAccounts() {
    m_mainWindow->getUi()->transferListWidget->clear();

    for (int i = 0; i < m_account->getSize(); i++) {
        Account::Data account = m_account->getAccount(i);

        QString display = QString::number(i + 1) + ")\t" + account.number + "\t" + account.name;
        m_mainWindow->getUi()->transferListWidget->addItem(display);
    }

}

void TransferPage::selectElement(bool amount) {
	QString select = "border: 1px solid rgb(64, 132, 214); border-radius: 5px;";
	QString deselect = "border: 1px solid rgb(48, 52, 64); border-radius: 5px;";

	if (amount) {
		m_mainWindow->getUi()->transferListWidget->setStyleSheet(deselect);
		m_mainWindow->getUi()->transferDisplay->setStyleSheet(select);
	}
	else {
		m_mainWindow->getUi()->transferListWidget->setStyleSheet(select);
		m_mainWindow->getUi()->transferDisplay->setStyleSheet(deselect);
	}
}

void TransferPage::selectAccount(int select) {
	m_mainWindow->getUi()->transferListWidget->setCurrentRow(select);
}

void TransferPage::numberEntered(qint64 amount) {
	m_mainWindow->getUi()->transferDisplay->setText(Account::formatCurrency(amount * FACTOR));
}

void TransferPage::transferAttempt(bool success, QString message) {
	if (!success) {
		m_mainWindow->getUi()->transferDisplay->setStyleSheet(
				"border: 1px solid red; border-radius: 5px;");
		m_mainWindow->getUi()->transferMessage->setText(message);
	}
	else {
		selectElement(true);
		m_mainWindow->getUi()->transferMessage->setText(message);
	}

	numberEntered(0);
}
