#include <QSqlQuery>
#include <QThread>
#include <QVariant>

#include "db.h"
#include "user.h"

User::User(QThread *thread) :
	m_id(0),
	m_firstName(""),
	m_lastName(""),
	m_role(0)
{
	this->moveToThread(thread);
	QObject::connect(thread,	&QThread::started,	this,	&User::process);
	QObject::connect(this,		&User::finished,	thread,	&QThread::quit);
	QObject::connect(this,		&User::finished,	this,	&User::deleteLater);
	QObject::connect(thread,	&QThread::finished,	thread,	&QThread::deleteLater);
}

User::~User() {
}

void User::process() {
}

void User::login(int userId) {
	QSqlQuery query = Db::q("SElECT * FROM `user` WHERE `id`=?", {userId});
	if (query.exec() && query.size() == 1 && query.next()) {
		m_mutex.lock();
		m_id = query.value("id").toInt();
		m_firstName = query.value("first_name").toString();
		m_lastName = query.value("last_name").toString();
		m_role = query.value("role").toInt();
		m_mutex.unlock();

		emit emitLogin();

#ifdef DEBUG
	printf("User, login: %d, %s %s\n",
			userId,
			query.value("first_name").toString().toUtf8().data(),
			query.value("last_name").toString().toUtf8().data());
#endif
		emit setAccounts(userId);
	}
}

int User::getId() {
	QMutexLocker locker(&m_mutex);
	return m_id;
}

QString User::getFirstName() {
	QMutexLocker locker(&m_mutex);
	return m_firstName;
}

QString User::getLastName() {
	QMutexLocker locker(&m_mutex);
	return m_lastName;
}

int User::getRole() {
	QMutexLocker locker(&m_mutex);
	return m_role;
}

void User::setId(int id) {
	m_mutex.lock();
	m_id = id;
	m_mutex.unlock();
}

void User::setFirstName(QString firstName) {
	m_mutex.lock();
	m_firstName = firstName;
	m_mutex.unlock();
}

void User::setLastName(QString lastName) {
	m_mutex.lock();
	m_lastName = lastName;
	m_mutex.unlock();
}

void User::setRole(int role) {
	m_mutex.lock();
	m_role = role;
	m_mutex.unlock();
}
